#ifndef INC_GTTAGS_H
#define INC_GTTAGS_H

#include <stdarg.h>
#include <string.h>

#include "gttypes.h"

/************************************************************************\
*
* Tag management module header.
*
* $Ident: This file is part of the 'Gt-library' 1.0.0 $
* $Copyright: (C) 1997 by Gran Gustafsson $
*
\************************************************************************/

/* Structure of a tag */
typedef struct GtTag
{
	ulong tag;
	union
	{
		ulong val;
		void *ptr;
	} d;
} GtTag;

/* Flag for all user tags */
#define GT_USER   0x80000000

/* Specail system tags */
#define GT_DONE     ((ulong)0x00000000)		/* Marks end of tags */
#define GT_MORE   					0x00000001		/* Points to more tags */
#define GT_SKIP   					0x00000002		/* Skip a number of tags */
#define GT_IGNORE 					0x00000003		/* Ignore tag */

extern GtTag *GtAllocTags(int nrtags);
extern void GtFreeTags(GtTag *tags);
extern GtTag *GtCloneTags(GtTag *tags);
extern GtTag *GtRefreshTags(GtTag *dest,GtTag *src);
extern GtTag *GtFindTag(GtTag *tags,ulong tag);
extern ulong GtGetTagVal(GtTag *tags,ulong tag,ulong def);
extern void *GtGetTagPtr(GtTag *tags,ulong tag,void *def);
extern GtTag *GtNextTag(GtTag **tags);
extern ulong GtPackFlagTags(GtTag *tags,GtTag *tagflags,ulong base);
extern void GtArrayTags(GtTag *tags,GtTag *tagoff,ulong *array,ubyte *found);
extern int GtMapTags(GtTag *tags,GtTag *map);

extern int GtBeginVaTags(void);
extern GtTag *GtCopyVaTags(ulong first,va_list *va);
extern void GtEndVaTags(int p);

#endif /* INC_GTTAGS_H */
